@@ -2,24 +2,12 @@ package Number::Phone;
 
 use strict;
 
-use File::ShareDir;
-use File::Spec::Functions qw(catfile);
-use File::Basename qw(dirname);
-use Cwd qw(abs_path);
-
 use Scalar::Util 'blessed';
 
-use Number::Phone::Country qw(noexport);
-use Number::Phone::Data;
+use Number::Phone::Country qw(noexport uk);
 use Number::Phone::StubCountry;
 
-use Devel::Deprecations::Environmental
-    Int32   => { unsupported_from => '2023-06-01' },
-    OldPerl => { unsupported_from => '2022-11-08', older_than => '5.10.0' },
-    OldPerl => { unsupported_from => '2023-01-08', older_than => '5.12.0' };
-
-# MUST be in format N.NNNN, see https://github.com/DrHyde/perl-modules-Number-Phone/issues/58
-our $VERSION = '3.8099_02';
+our $VERSION = '3.0001';
 
 my $NOSTUBS = 0;
 sub import {
@@ -30,59 +18,67 @@ sub import {
   }
 }
 
-sub _find_data_file {
-    my $wanted = shift;
-
-    # giant ball of hate because lib::abs doesn't work on Windows
-    my $this_file = __FILE__;
-    my $this_dir  = dirname($this_file);
-
-    my @candidate_files = (
-         # if this is $devdir/lib ...
-         catfile($this_dir, qw(.. .. lib .. share), $wanted),
-         # if this is $devdir/blib/lib ...
-         catfile($this_dir, qw(.. .. .. blib lib .. .. share), $wanted),
-         # if this has been installed
-         catfile(File::ShareDir::dist_dir('Number-Phone'), $wanted),
-    );
-    my $file = (grep { -e $_ } @candidate_files)[0];
-
-    if(!$file) {
-        die(
-            "Couldn't find data file '$wanted' amongst:\n".
-            join('', map { "  $_\n" } @candidate_files)
-        );
-    }
-    return $file;
-}
 
 my @is_methods = qw(
     is_valid is_allocated is_in_use
     is_geographic is_fixed_line is_mobile is_pager
     is_tollfree is_specialrate is_adult is_network_service is_personal
     is_corporate is_government is_international
-    is_ipphone is_isdn is_drama
+    is_ipphone is_isdn
 );
 
 foreach my $method (
     @is_methods, qw(
         country_code regulator areacode areaname
         subscriber operator operator_ported translates_to
-        format location data_source
+        format location
     )
 ) {
     no strict 'refs';
-    *{__PACKAGE__."::$method"} = sub { undef; }
+    *{__PACKAGE__."::$method"} = sub {
+        my $self = shift;
+        warn("DEPRECATION: ".__PACKAGE__."->$method should only be called as an object method\n")
+          unless(blessed($self));
+        return undef if(blessed($self) && $self->isa(__PACKAGE__));
+        $self = shift if(
+            $self eq __PACKAGE__ ||
+            substr($self, 0, 2 + length(__PACKAGE__)) eq __PACKAGE__.'::'
+        );
+        $self = __PACKAGE__->new($self)
+            unless(blessed($self) && $self->isa(__PACKAGE__));
+        return $self->$method() if($self);
+        undef;
+    }
 }
 
 sub type {
-    my $self = shift;
-    my $rval = [grep { $self->$_() } @is_methods];
+    my $parm = shift;
+    warn("DEPRECATION: ".__PACKAGE__."->type should only be called as an object method\n")
+      unless(blessed($parm));
+    my $class = __PACKAGE__;
+
+    no strict 'refs';
+
+    unless(blessed($parm) && $parm->isa(__PACKAGE__)) {
+        if(
+            $parm eq __PACKAGE__ ||
+            substr($parm, 0, 2 + length(__PACKAGE__)) eq __PACKAGE__.'::'
+        ) {
+            $class = $parm;
+            $parm = shift;
+        }
+        $parm = $class->new($parm);
+    }
+
+    my $rval = $parm ?
+        [grep { $parm->$_() } @is_methods] :
+        undef;
     wantarray() ? @{$rval} : $rval;
 }
 
 sub country {
     my $class = blessed(shift);
+    return unless $class;
     (my @two_letter_codes) = $class =~ /\b([A-Z]{2})\b/g;
     return $two_letter_codes[-1];
 }
@@ -93,20 +89,17 @@ sub dial_to {
 
   if($from->country_code() != $to->country_code()) {
     return Number::Phone::Country::idd_code($from->country()).
-        $to->country_code().
-        ($to->isa('Number::Phone::StubCountry')
-          ? $to->raw_number()
-          : (
-              ($to->areacode() ? $to->areacode() : '').
-              $to->subscriber()
-            )
-        );
+           $to->country_code().
+           ($to->areacode() ? $to->areacode() : '').
+           $to->subscriber();
   }
 
+  # if we get here it's a domestic call
+
   # do we know how to do this?
   my $intra_country_dial_to = eval '$from->intra_country_dial_to($to)';
   return undef if($@); # no
-  return $intra_country_dial_to if(defined($intra_country_dial_to)); # yes, and here's how
+  return $intra_country_dial_to if($intra_country_dial_to); # yes, and here's how
 
   # if we get here, then we can use the default implementation ...
 
@@ -123,27 +116,6 @@ sub dial_to {
 
 sub intra_country_dial_to { die("don't know how\n"); }
 
-sub format_using {
-    my $self   = shift;
-    my $format = shift;
-
-    return $self->format() if($format eq 'E123');
-
-    eval "use Number::Phone::Formatter::$format";
-    die("Couldn't load format '$format': $@\n") if($@);
-    return "Number::Phone::Formatter::$format"->format($self->format(), $self);
-}
-
-sub format_for_country {
-  my $self = shift;
-  my $country_code = shift || '';
-  $country_code = Number::Phone::Country::country_code($country_code)
-    if $country_code && $country_code =~ /[a-z]/i;
-  $country_code =~ s/^\+//;
-  return $self->format_using('National') if $country_code eq $self->country_code();
-  return $self->format_using('NationallyPreferredIntl');
-}
-
 1;
 
 =head1 NAME
@@ -176,52 +148,17 @@ so the appropriate country-specific module is loaded if available.
 If you pass in a bogus country code not recognised by
 Number::Phone::Country, the constructor will return undef.
 
-=head1 INCOMPATIBLE CHANGES
-
-Early versions of this module allowed what are now object methods
-to also be called as class methods or even as functions. This was a
-bad design decision. Use of those calling conventions was deprecated
-in version 2.0, released in January 2012, and started to emit
-warnings. All code to support those calling conventions has now been removed.
-
-Until 2017 we used KOS for the country code for Kosovo, that has now changed to
-XK. See L<Number::Phone::Country>.
-
-From version 3.4000 to 3.4003 inclusive we accepted any old garbage after
-+383 as being valid, as the Kosovo numbering plan had not been published.
-Now that that has been published, we use libphonenumber data, and validate
-against it.
-
-The prefix codes in 3.4003 and earlier were managed by hand and so got out
-of date. After that release they are mostly derived from libphonenumber.
-libphonenumber's data includes carrier selection codes when they are
-mandatory for dialling so those are now included. This sometimes means that
-some random carrier has been arbitrarily privileged over others.
-
-As of version 3.6000 the C<areaname> method is documented as taking an optional
-language code. As far as I can tell providing this new parameter to the method
-as provided by all the subclasses on the CPAN won't do any harm.
-
-As of version 3.6000 the C<areaname> method pays attention to your locale
-settings and so you might start getting locale-appropriate versions of
-areanames instead of what you used to get.
-
-3.8000 is a bit stricter about numbers and countries not matching in the
-constructor. This may affect users who specify places like Guernsey but
-provide numbers from Jersey or the Isle of Man, all three of which are separate
-jurisdictions squatting on random places all over the UK's number plan.
-
-=head1 COMPATIBILITY WITH libphonenumber
+=head1 COMPATIBILTY WITH libphonenumber
 
 libphonenumber is a similar project for other languages, maintained
-by Google.
+by Google. 
 
 If you pass in a country code for which
 no supporting module is available, the constructor will try to use a 'stub'
 class under Number::Phone::StubCountry::* that uses data automatically
 extracted from Google's libphonenumber project.  libphonenumber doesn't
-have enough data to support all the features of Number::Phone.
-If you want to disable this, then pass 'nostubs'
+have enough data to support all the features of Number::Phone, and this
+is an experimental feature.  If you want to disable this, then pass 'nostubs'
 when you use the module:
 
     use Number::Phone qw(nostubs);
@@ -230,84 +167,34 @@ Alternatively, if you want to *always* use data derived from libphonenumber,
 you should use the L<Number::Phone::Lib> module instead. This is a subclass
 of Number::Phone that will use the libphonenumber-derived stub classes even
 when extra data is available in, for example, Number::Phone::UK. You might
-want to do this for compatibility or performance. Number::Phone::UK is quite
-slow, because it uses a huge database for some of its features.
-
-=head1 PERL VERSIONS SUPPORTED
-
-A perl built with support for 64 bit ints will be required some time after 2023-06-01.
-This is the default for all vaguely recent builds of perl on all modern systems.
-
-Perl 5.8 (and below) is not fully supported as of version 3.8006. This is
-because I have a dependency when packaging it on L<XML::XPath> which dropped
-support for perl 5.8. This means that I can no longer automatically test
-Number::Phone on perl 5.8. There is no runtime dependency on XML::XPath, so I
-expect Number::Phone to keep on truckin' on 5.8, and I won't B<deliberately>
-remove support without notice. However, because I can not automatically test, I
-can't guarantee that I won't B<accidentally> remove support. If I do
-accidentally remove it, it ain't coming back.
-
-Similarly, Perl 5.10 is not fully supported as of the version after 3.8007, whatever
-that may be. This is because there is a dependency on L<Test::Deep>, which
-dropped support for that version of perl at the start of 2023. This means that
-I can no longer automatically test on that version of perl, and you can't
-(easily) install the necessary dependencies. Depending on details of what you
-installed when, future releases of Number::Phone may or may not continue to
-work for you.
+want to do this for compatibility or performance. Number::Phone::UK is quite slow, because it uses a huge database for some of its features.
 
 =cut
 
 sub _new_args {
     my $class = shift;
     my($country, $number) = @_;
-    die("Number::Phone->new(): too many params\n") if(exists($_[2]));
-
-    my $original_country;
-    $original_country = $country if($country =~ /::/);
 
     if(!defined($number)) { # one arg
-        $number = $country;
-    } elsif($country =~ /[a-z]/i) {
-        # eg ('UK', '12345')
-        #    ('MOCK', ...)
-        #    ('InternationalNetworks882', ...)
-        # we accept lower-case ISO codes
-        $original_country = uc($country) if($country =~ /^[a-z]{2}$/i);
-        if($country =~ /^GMSS::[a-zA-Z]+$/) {
-            if($number !~ /^\+881/) {
-                $number = "+881$number";
-            }
-        } elsif($country =~ /^InternationalNetworks(88[23])::[a-zA-Z]+$/) {
-            my $idd = $1;
-            if($number !~ /^\+$idd/) {
-                $number = "+$idd$number";
-            }
-        } elsif(index($number, '+'.Number::Phone::Country::country_code($country)) != 0) {
-            $number = '+'.Number::Phone::Country::country_code($country).$number;
-        }
+      $number = $country;
+    } elsif($country =~ /[a-z]/i) { # eg 'UK', '12345'
+      $number = '+'.
+                Number::Phone::Country::country_code($country).
+		$number
+        unless(index($number, '+'.Number::Phone::Country::country_code($country)) == 0);
     } else { # (+)NNN
-        $number = join('', grep { defined } ($country, $number));
+      $number = join('', grep { defined } ($country, $number));
     }
 
+    die("Need to specify a number for ".__PACKAGE__."->new()\n")
+        unless($number);
+    die("Number::Phone->new(): too many params\n")
+        if(exists($_[2]));
     $number =~ s/[^+0-9]//g;
+
     $number = "+$number" unless($number =~ /^\+/);
-
     $country = Number::Phone::Country::phone2country($number) or return;
-    if($country eq 'AQ' && $number =~ /^\+882/) {
-        $original_country = 'InternationalNetworks882';
-    }
-
-    # special cases where you can legitimately ask for a containing country (eg
-    # GB) and get back a sub-country (eg GG, which squats upon parts of the GB
-    # number plan)
-    if(
-        ($country eq 'VA'           && $original_country eq 'IT') ||
-        ($country =~ /^(IM|GG|JE)$/ && $original_country eq 'GB')
-    ) {
-        $original_country = $country;
-    }
-
-    return ($original_country || $country), $number;
+    return $country, $number;
 }
 
 sub new {
@@ -316,60 +203,43 @@ sub new {
     return undef unless($country);
     if ($number =~ /^\+1/) {
         $country = "NANP";
-    } elsif($country eq 'GB') {
-        # for hysterical raisins
+    } elsif ($country =~ /^(?:GG|JE|IM)$/) {
         $country = 'UK';
-    } elsif($country =~ /^(GG|JE|IM)$/) {
-        $country = "UK::$country";
     }
     eval "use Number::Phone::$country";
     if($@ || !"Number::Phone::$country"->isa('Number::Phone')) {
-        if($@ =~ /--without_uk/) {
-            # a test unexpectedly tried to load Number::Phone::UK, argh!
-            die $@
-        }
-        # undo the above transformations, for stub-land
-        if($country eq 'UK') { $country = 'GB' }
-        if($country =~ /^UK::(..)/) { $country = $1 }
         return $class->_make_stub_object($number, $country)
     }
     return "Number::Phone::$country"->new($number);
 }
 
 sub _make_stub_object {
-    my ($class, $number, $country_name) = @_;
-    die("no module available for $country_name, and nostubs turned on\n") if($NOSTUBS);
-
-    my $stub_class = "Number::Phone::StubCountry::$country_name";
-    eval "use $stub_class";
-    if($@ && $number =~ /^\+881/) {
-        $stub_class = "Number::Phone::StubCountry::GMSS::$country_name";
-        eval "use $stub_class";
-    } elsif($@ && $number =~ /^\+882/ && $country_name ne 'AQ') {
-        $stub_class = "Number::Phone::StubCountry::InternationalNetworks882::$country_name";
-        eval "use $stub_class";
-    } elsif($@ && $number =~ /^\+883/) {
-        $stub_class = "Number::Phone::StubCountry::InternationalNetworks883::$country_name";
-        eval "use $stub_class";
-    } elsif($@) {
-        my (undef, $country_idd) = Number::Phone::Country::phone2country_and_idd($number);
-        # an instance of this class is the ultimate fallback
-        (my $local_number = $number) =~ s/(^\+$country_idd|\D)//;
-        if($local_number eq '') { return undef; }
-        return bless({
-            country_code => $country_idd,
-            # country      => $country_name,
-            is_valid     => undef,
-            number       => $local_number,
-        }, 'Number::Phone::StubCountry');
-    }
-
-    $stub_class->new($number);
+ my ($class, $number, $country_name) = @_;
+  die("no module available for $country_name, and nostubs turned on\n") if($NOSTUBS);
+  my $stub_class = "Number::Phone::StubCountry::$country_name";
+  eval "use $stub_class";
+  # die("Can't find $stub_class: $@\n") if($@);
+  if($@) {
+      my (undef, $country_idd) = Number::Phone::Country::phone2country_and_idd($number);
+      # an instance of this class is the ultimate fallback
+      (my $local_number = $number) =~ s/(^\+$country_idd|\D)//;
+      return bless({
+          country_code => $country_idd,
+          country      => $country_name,
+          is_valid     => undef,
+          number       => $local_number,
+      }, 'Number::Phone::StubCountry');
+  }
+  $stub_class->new($number);
 }
 
 =head1 METHODS
 
-All Number::Phone classes can implement the following object methods.
+All Number::Phone classes can implement the following methods, as
+object methods.  Note that in previous versions these were also required
+to work as class methods and could also work as subroutines.  That
+was a bad design decision and is deprecated.  Number::Phone will spit
+warnings if you try that now, and support will be removed in the future.
 
 The implementations in the parent class all return undef unless otherwise
 noted.
@@ -478,11 +348,6 @@ various "Country Direct"-a-likes.  See also the C<country()> method.
 The number is some kind of network service such as the operator, directory
 enquiries, emergency services etc
 
-=item is_drama
-
-The number is for use in fiction, such as TV and Radio drama programmes.
-It will not be allocated for use in real life.
-
 =back
 
 =head2 OTHER NUMBER METADATA METHODS
@@ -514,13 +379,6 @@ the superclass definition returns undef.  For instance, for a number
 beginning +44 20 it would return
 'London'.  Note that this may return data in non-ASCII character sets.
 
-This may take an optional language code such as 'de' or 'en'. If
-you provide that then you will get back whatever the place name is
-in that language, if the data is available. If you don't provide
-it then it will first look at your locale settings and try to find
-a name in an appropriate language, and if nothing is found fall
-back to English.
-
 =item location
 
 This returns an approximate geographic location for the number if possible.
@@ -568,37 +426,12 @@ C<[qw(valid allocated geographic)]>.
 
 =item format
 
-Return a sanely formatted E.123-compliant version of the number, complete with
-IDD code, eg for the UK number (0208) 771-2924 it would return +44 20 8771
-2924.
+Return a sanely formatted version of the number, complete with IDD code, eg
+for the UK number (0208) 771-2924 it would return +44 20 8771 2924.
 
 The superclass implementation returns undef, which is nonsense, so you
 should always implement this.
 
-=item format_using
-
-If you want something different from E.123, then pass this the name of a
-L<formatter|Number::Phone::Formatters> to use.
-
-For example, if you want to get "just the digits, ma'am", use the
-L<Raw|Number::Phone::Formatter::Raw> formatter thus:
-
-  Number::Phone->new('+44 20 8771 2924')->format_using('Raw');
-
-which will return:
-
-  2087712924
-
-It is a fatal error to specify a non-existent formatter.
-
-=item format_for_country
-
-Given a country code (either two-letter ISO or numeric prefix), return the
-number formatted either nationally-formatted, if the number is in the same
-country, or as a nationally-preferred international number if not. Internally
-this uses the National and NationallyPreferredIntl formatters. Beware of the
-potential performance hit!
-
 =item country
 
 The two letter ISO country code for the country in which the call will
@@ -630,34 +463,14 @@ The superclass implementation returns undef.
 
 =back
 
-=head2 DATA SOURCES
-
-=over
-
-=item data_source
-
-Class method, return some hopefully useful text about the source of the data
-(if any) that drives a country-specific module. The implementation in the base
-class returns undef as the base class itself has no data source.
-
-=item libphonenumber_tag
-
-Class method which you should not over-ride, implemented in the base class.
-Returns the version of libphonenumber whose metadata was used for this release
-of Number::Phone. NB that this is derived from their most recent git tag, so
-may occasionally be a little ahead of the most recent libphonenumber release as
-the tag gets created before their release is built.
-
-The current version of this is also documented in L<Number::Phone::Data>.
-
-=back
-
 =head2 HOW TO DIAL FROM ONE NUMBER TO ANOTHER
 
 =over
 
 =item dial_to
 
+EXPERIMENTAL METHOD
+
 Takes another Number::Phone object as its only argument and returns a
 string showing how to dial from the number represented by the invocant
 to that represented by the argument.
@@ -693,8 +506,8 @@ follows:
 =item international call
 
 Append together the source country's international dialling prefix
-(usually 00), then the destination country's country code, area code
-(if the country has such a thing), and subscriber number.
+(usually 00), then the destination country's code code, area code,
+and subscriber number.
 
 =item domestic call, different area code
 
@@ -751,39 +564,13 @@ the phone number is passed to its constructor unchanged.
 If only one parameter is passed, then we try to figure out which is the right
 country subclass to use by pre-pending a + sign to the number if
 there isn't one, and looking the country up using
-Number::Phone::Country. That gives us a two letter country code that
-is used to try to load the right module. We then pass the number through to
-that module's constructor and return whatever it says (which may be undef
-if you pass in an invalid number - see SUBCLASSING below).
+Number::Phone::Country.  That gives us a two letter country code that
+is used to try to load the right module.
 
 The constructor returns undef if it can not figure out what country
 you're talking about, or an object based on Google's libphonenumber
 data if there's no complete country-specific module available.
 
-It is generally assumed that numbers are complete and unambiguous - ie you
-can't normally pass just the local part to the constructor if the number has an
-area code. Any subclass's constructor which contravenes this should document
-it.
-
-If you call it with two parameters, then the two must match. ie, if you
-do this:
-
-    Number::Phone->new("FR", "+441424220001")
-
-you will get C<undef> back because whiel the number is valid, it ain't French.
-This usually applies to the case where a single country's number plan contains
-other jurisdictions, such as the case of Guernsey, Jersey and the Isle of Man
-squatting on the United Kingdom's number plan. For example, this fails, because
-the number is from Guernsey, not Jersey:
-
-    Number::Phone->new('JE', '01481256789')
-
-For backward compatibility and convenience, however, if you ask for an object
-representing a number in the "host" country but pass a number for the
-"sub-country" then you'll get back a valid object representing the sub-country:
-
-    my $gg_number = Number::Phone->new('GB', '01481256789')
-
 =back
 
 =head1 SUBCLASSING
@@ -806,35 +593,16 @@ FR and Ireland would be IE.  As usual, the UK is an exception, using UK
 instead of the ISO-mandated GB.  NANP countries are also an exception,
 going like Number::Phone::NANP::XX.
 
-=head1 UPDATES
-
-I release updates approximately every three months, including new data.
-
-I will also do intercalary releases to fix *serious* bugs in the code
-and when *large* data updates (eg when a country's numbering scheme changes)
-are brought to my attention.
-
-I will not normally do a release just because a country has added some
-new number range. If this irks you then I would welcome a discussion on
-how you can best write a patch, with tests, that will reliably incorporate
-updated data from libphonenumber. Much of the needed code already exists
-in the repository but it is not fit for end-user consumption.
-
 =head1 BUGS/FEEDBACK
 
 Please report bugs by at L<https://github.com/DrHyde/perl-modules-Number-Phone/issues>, including, if possible, a test case.
 
-=head1 MAILING LIST
-
-There is a mailing list for discussion and help. Please subscribe at
-L<http://lists.scsys.co.uk/cgi-bin/mailman/listinfo/number-phone>.
-
-Kindly hosted by L<Shadowcat|http://scsys.co.uk>.
+I welcome feedback from users.
 
 =head1 SEE ALSO
 
-L<https://github.com/googlei18n/libphonenumber>, a similar project for Java,
-C++ and Javascript. Number::Phone imports its data.
+L<http://code.google.com/p/libphonenumber/>, a similar project for Java,
+C++ and Javascript
 
 =head1 SOURCE CODE REPOSITORY
 
@@ -842,7 +610,7 @@ L<git://github.com/DrHyde/perl-modules-Number-Phone.git>
 
 =head1 AUTHOR, COPYRIGHT and LICENCE
 
-Copyright 2004 - 2015 David Cantrell E<lt>F<david@cantrell.org.uk>E<gt>
+Copyright 2004 - 2012 David Cantrell E<lt>F<david@cantrell.org.uk>E<gt>
 
 This software is free-as-in-speech software, and may be used,
 distributed, and modified under the terms of either the GNU
