@@ -7,7 +7,7 @@ use Number::Phone::UK::Data;
 
 use base 'Number::Phone';
 
-our $VERSION = 1.66;
+our $VERSION = 1.63;
 
 my $cache = {};
 
@@ -26,45 +26,19 @@ Number::Phone::UK - UK-specific methods for Number::Phone
 sub new {
     my $class = shift;
     my $number = shift;
+    die("No number given to ".__PACKAGE__."->new()\n") unless($number);
 
-    $number = '+44'._clean_number($number);
     if(is_valid($number)) {
-        $number =~ s/^0/+44/;
         return bless(\$number, $class->_get_class(_clean_number($number)));
     } else { return undef; }
 }
 
-=head1 DATABASE
-
-Number::Phone::UK uses a large database, access via L<Number::Phone::UK::Data>. This
-database lives in a file, and normally only the little bits of it that you access will
-ever get loaded into memory. This means, however, that creating Number::Phone::UK objects
-almost always involves disk access and so is slow compared to data for some other
-countries. There are two ways to avoid this slowness.
-
-First, if you don't need all the functionality you can use L<Number::Phone::Lib>.
-
-Second, if you can accept slow startup - eg when your server starts - then you can call
-C<Number::Phone::UK::Data->slurp()> from your code, which will pull the entire database
-into memory. This will take a few minutes, and on a 64-bit machine will consume of the
-order of 200MB of memory.
-
-The database uses L<DBM::Deep>. This apparently has some problems if you connect to it,
-C<fork()>, and then try to access the database from multiple processes. We attempt to
-work around this by re-connecting to the database after forking. This is, of course,
-not a problem if you C<slurp()> the database before forking.
-
 =head1 METHODS
 
 The following methods from Number::Phone are overridden:
 
 =over 4
 
-=item new
-
-The constructor, you should never have to call this yourself. To create an
-object the canonical incantation is C<Number::Phone->new('+44 ...')>.
-
 =item is_valid
 
 The number is valid within the national numbering scheme.  It may or may
@@ -76,10 +50,11 @@ true for any of the following methods will also be valid.
 sub _get_class {
   my $class = shift;
   my $number = shift;
-  foreach my $prefix (_prefixes($number)) {
-    if(exists(Number::Phone::UK::Data::db()->{subclass}->{$prefix})) {
-      my $subclass = join('::', $class, Number::Phone::UK::Data::db()->{subclass}->{$prefix});
+  foreach my $prefix (_retards($number)) {
+    if(exists($Number::Phone::UK::Data::db->{subclass}->{$prefix})) {
+      my $subclass = join('::', $class, $Number::Phone::UK::Data::db->{subclass}->{$prefix});
       eval "use $subclass";
+      die($@) if($@);
       return $subclass;
     }
   }
@@ -94,7 +69,7 @@ sub _clean_number {
     return $clean;
 }
 
-sub _prefixes {
+sub _retards {
     my $number = shift;
     map { substr($number, 0, $_) } reverse(1..length($number));
 }
@@ -105,7 +80,7 @@ sub is_valid {
     # If called as an object method, it *must* be valid otherwise the
     # object would never have been instantiated.
     # If called as a subroutine, that's the constructor doing its thang.
-    return 1 if(blessed($number));
+    return 1 if(blessed($number) && $number->isa(__PACKAGE__));
 
     # otherwise we have to validate
 
@@ -117,7 +92,7 @@ sub is_valid {
 
     my $cleaned_number = _clean_number($number);
 
-    my @prefixes = _prefixes($cleaned_number);
+    my @retards = _retards($cleaned_number);
 
     # quickly check length
     return $cache->{$number}->{is_valid} = 0 if(length($cleaned_number) < 7 || length($cleaned_number) > 10);
@@ -127,14 +102,12 @@ sub is_valid {
     return $cache->{$number}->{is_valid} = 0 if($cleaned_number =~ /^([27]|11)/ && length($cleaned_number) != 10);
 
     $cache->{$number}->{is_allocated} = 
-        grep { Number::Phone::UK::Data::db()->{telco_and_length}->{$_} } @prefixes;
-
+        grep { $Number::Phone::UK::Data::db->{telco_and_length}->{$_} } @retards;
     if($cache->{$number}->{is_allocated}) {
-        my($telco_and_length) = map { Number::Phone::UK::Data::db()->{telco_and_length}->{$_} } grep { Number::Phone::UK::Data::db()->{telco_and_length}->{$_} } @prefixes;
-        $cache->{$number}->{operator} = Number::Phone::UK::Data::db()->{telco_format}->{$telco_and_length}->{telco};
-        $cache->{$number}->{format} = Number::Phone::UK::Data::db()->{telco_format}->{$telco_and_length}->{format};
-
-        if($cache->{$number}->{format} =~ /\+/) {
+        my($telco_and_length) = map { $Number::Phone::UK::Data::db->{telco_and_length}->{$_} } grep { $Number::Phone::UK::Data::db->{telco_and_length}->{$_} } @retards;
+        $cache->{$number}->{operator} = $Number::Phone::UK::Data::db->{telco_format}->{$telco_and_length}->{telco};
+        $cache->{$number}->{format} = $Number::Phone::UK::Data::db->{telco_format}->{$telco_and_length}->{format};
+        if(defined($cache->{$number}->{format}) && $cache->{$number}->{format} =~ /\+/) {
             my($arealength, $subscriberlength) = split(/\+/, $cache->{$number}->{format});
             # for hateful mixed thing
             my @subscriberlengths = ($subscriberlength =~ m{/}) ? split(/\//, $subscriberlength) : ($subscriberlength);
@@ -143,8 +116,8 @@ sub is_valid {
             $cache->{$number}->{subscriber} = substr($cleaned_number, $arealength);
             $cache->{$number}->{areaname} = (
                 map {
-                    Number::Phone::UK::Data::db()->{areanames}->{$_}
-                } grep { Number::Phone::UK::Data::db()->{areanames}->{$_} } @prefixes
+                    $Number::Phone::UK::Data::db->{areanames}->{$_}
+                } grep { $Number::Phone::UK::Data::db->{areanames}->{$_} } @retards
             )[0];
             if(!grep { length($cache->{$number}->{subscriber}) == $_ } @subscriberlengths) {
                 # number wrong length!
@@ -162,45 +135,6 @@ sub is_fixed_line {
   return undef;
 }
 
-sub is_drama {
-    my $self = shift;
-
-    my $num = _clean_number(${$self});
-
-    my @drama_numbers = (
-        # Leeds, Sheffield, Nottingham, Leicester, Bristol, Reading
-        qr/^11[3-8]4960[0-9]{3}$/,
-        # Birmingham, Edinburgh, Glasgow, Liverpool, Manchester
-        qr/^1[2-6]14960[0-9]{3}$/,
-        # London
-        qr/^2079460[0-9]{3}$/,
-        # Tyneside/Durham/Sunderland
-        qr/^1914980[0-9]{3}$/,
-        # Northern Ireland
-        qr/^2896496[0-9]{3}$/,
-        # Cardiff
-        qr/^2920180[0-9]{3}$/,
-        # No area
-        qr/^1632960[0-9]{3}$/,
-        # Mobile
-        qr/^7700900[0-9]{3}$/,
-        # Freephone
-        qr/^8081570[0-9]{3}$/,
-        # Premium Rate
-        qr/^9098790[0-9]{3}$/,
-        # UK Wide
-        qr/^3069990[0-9]{3}$/,
-    );
-
-    foreach my $d (@drama_numbers) {
-        
-        return 1 if ($num =~ $d);
-    
-    }
-
-    return 0;
-}
-
 foreach my $is (qw(
     geographic network_service tollfree corporate
     personal pager mobile specialrate adult allocated ipphone
@@ -208,10 +142,15 @@ foreach my $is (qw(
     no strict 'refs';
     *{__PACKAGE__."::is_$is"} = sub {
         my $self = shift;
+        warn("DEPRECATION: ".__PACKAGE__."->is_$is should only be called as an object method\n")
+          unless(blessed($self));
+        $self = shift if($self eq __PACKAGE__);
+        $self = __PACKAGE__->new($self)
+            unless(blessed($self) && $self->isa(__PACKAGE__));
         if(!exists($cache->{${$self}}->{"is_$is"})) {
           $cache->{${$self}}->{"is_$is"} = 
             grep {
-              Number::Phone::UK::Data::db()->{
+              $Number::Phone::UK::Data::db->{
                 { geographic      => 'geo_prefices',
                   network_service => 'network_svc_prefices',
                   tollfree        => 'free_prefices',
@@ -224,7 +163,7 @@ foreach my $is (qw(
                   ipphone         => 'ip_prefices'
                 }->{$is}
               }->{$_}
-            } _prefixes(_clean_number(${$self}));
+            } _retards(_clean_number(${$self}));
         }
         $cache->{${$self}}->{"is_$is"};
     }
@@ -236,6 +175,11 @@ foreach my $method (qw(operator areacode areaname subscriber)) {
     no strict 'refs';
     *{__PACKAGE__."::$method"} = sub {
         my $self = shift;
+        warn("DEPRECATION: ".__PACKAGE__."->$method should only be called as an object method\n")
+          unless(blessed($self));
+        $self = (blessed($self) && $self->isa(__PACKAGE__)) ?
+            $self :
+            __PACKAGE__->new($self);
         return $cache->{${$self}}->{$method};
     }
 }
@@ -245,13 +189,6 @@ foreach my $method (qw(operator areacode areaname subscriber)) {
 The number has been allocated to a telco for use.  It may or may not yet
 be in use or may be reserved.
 
-=item is_drama
-
-The number is intended for use in fiction. OFCOM has allocated numerous small
-ranges for this purpose. These numbers will not be allocated to real customers.
-See L<http://stakeholders.ofcom.org.uk/telecoms/numbering/guidance-tele-no/numbers-for-drama>
-for the authoritative source.
-
 =item is_geographic
 
 The number refers to a geographic area.
@@ -335,20 +272,22 @@ that number is assigned, if available.  Otherwise returns undef.
 
 sub location {
     my $self = shift;
+    warn("DEPRECATION: ".__PACKAGE__."->location should only be called as an object method\n")
+      unless(blessed($self));
+    $self = (blessed($self) && $self->isa(__PACKAGE__)) ?
+        $self :
+        __PACKAGE__->new($self);
 
     return undef unless($self->is_geographic());
 
     my $cleaned_number = _clean_number(${$self});
 
-    my @prefixes = _prefixes($cleaned_number);
+    my @retards = _retards($cleaned_number);
 
-    # uncoverable branch true
-    if(!$ENV{TESTINGKILLTHEWABBIT}) {
-        eval "require Number::Phone::UK::DetailedLocations"; # uncoverable statement
-    }
+    eval "require Number::Phone::UK::DetailedLocations" unless($ENV{TESTINGKILLTHEWABBIT});
     require Number::Phone::UK::Exchanges if(!$Number::Phone::UK::Exchanges::db);
 
-    foreach(@prefixes) {
+    foreach(@retards) {
         if(exists($Number::Phone::UK::Exchanges::db->{exchg_prefices}->{$_})) {
             return [
                 $Number::Phone::UK::Exchanges::db->{exchg_positions}->{$Number::Phone::UK::Exchanges::db->{exchg_prefices}->{$_}}->{lat},
@@ -356,9 +295,8 @@ sub location {
             ];
         }
     }
-    # may become coverable if I ever test the location of a number
-    # in an areacode that wasn't in the data dump I got years ago
-    return undef; # uncoverable statement
+
+    return undef;
 }
 
 =item subscriber
@@ -374,24 +312,30 @@ number.
 =item format
 
 Return a sanely formatted version of the number, complete with IDD code, eg
-for the UK number (0208) 771-2924 it would return +44 20 8771 2924.
+for the UK number (0208) 771-2924 it would return +44 20 87712924.
 
 =cut
 
 sub format {
     my $self = shift;
-    my $r = ${$self};
-    if($self->areacode()) { # if there's an areacode ...
-        $r = '+'.country_code().' '.$self->areacode().' ';
-        if(    length($self->subscriber()) == 7) { $r .= substr($self->subscriber(), 0, 3).' '.substr($self->subscriber(), 3) }
-         elsif(length($self->subscriber()) == 8) { $r .= substr($self->subscriber(), 0, 4).' '.substr($self->subscriber(), 4) }
-         else                                    { $r .= $self->subscriber() }
-    } elsif(!$self->is_allocated()) { # if not allocated ...
-        $r = '+'.country_code().' '.substr(${$self}, 3)
-    } elsif($self->subscriber()) { # if there's a subscriber ...
-        $r = '+'.country_code().' '.$self->subscriber
-    }
-    return $r;
+    warn("DEPRECATION: ".__PACKAGE__."->format should only be called as an object method\n")
+      unless(blessed($self));
+    $self = (blessed($self) && $self->isa(__PACKAGE__)) ?
+        $self :
+        __PACKAGE__->new($self);
+    return (
+	# if there's an areacode ...
+        $self->areacode()      ? ('+'.country_code().' '.$self->areacode().' '.(
+          length($self->subscriber()) == 7 ? substr($self->subscriber(), 0, 3).' '.substr($self->subscriber(), 3) :
+          length($self->subscriber()) == 8 ? substr($self->subscriber(), 0, 4).' '.substr($self->subscriber(), 4) :
+                                             $self->subscriber() )) : 
+	# if not allocated ...
+        !$self->is_allocated() ? '+'.country_code().' '.( ${$self} =~ /^\+44/ ? substr(${$self}, 3) : substr(${$self}, 1)) :
+	# if there's a subscriber ...
+        $self->subscriber() ? '+'.country_code().' '.$self->subscriber :
+	# otherwise ...
+	  ${$self}
+    );
 }
 
 =item intra_country_dial_to
